﻿using System.IO;
using System.IO.IsolatedStorage;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using Microsoft.Devices;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework.Media;

namespace Camera
{
    public partial class MainPage : PhoneApplicationPage
    {
        public MainPage()
        {
            InitializeComponent();
            
        }

        PhotoCamera _camera;

        // ページがフレームでアクティブになったら呼び出される
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            _camera = new PhotoCamera();
            _camera.Initialized += CameraInitialized;
            PreviewBrush.SetSource(_camera);
        }

        // ページがフレームでアクティブでなくなったら呼び出される
        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            if (_camera == null) return;
            _camera.Dispose();
            _camera = null;
        }

        // カメラの初期化処理の完了
        void CameraInitialized(object sender, CameraOperationCompletedEventArgs e)
        {
            // 初期化処理に失敗した場合は何もしない
            if (!e.Succeeded) return;
            _camera.FlashMode = FlashMode.Off;

            // カメラの回転角度に合わせてプレビュー表示も回転させる
            Dispatcher.BeginInvoke(() =>
            {
                PreviewBrush.RelativeTransform = new CompositeTransform()
                {
                    CenterX = 0.5,
                    CenterY = 0.5,
                    Rotation = _camera.Orientation
                };
            });
        }

        private void PictureButton_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            // プレビューフレームを取得するバッファーを確保する
            double pixelSize = _camera.PreviewResolution.Width * _camera.PreviewResolution.Height;
            var pixelData = new int[(int)pixelSize];

            // プレビューフレームを取得
            _camera.GetPreviewBufferArgb32(pixelData);

            //Imageに表示
            // バッファーをWriteableBitmapへコピー
            var bmp = new WriteableBitmap((int)_camera.PreviewResolution.Width, (int)_camera.PreviewResolution.Height);
            pixelData.CopyTo(bmp.Pixels, 0);

            var stream = new MemoryStream();
            bmp.SaveJpeg(stream, bmp.PixelWidth, bmp.PixelHeight, 0, 80);
            using (var medialib = new MediaLibrary())
                medialib.SavePicture("preview.jpg", stream.ToArray());
        }

        private void IsolatedButton_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            // プレビューフレームを取得するバッファーを確保する
            double pixelSize = _camera.PreviewResolution.Width * _camera.PreviewResolution.Height;
            var pixelData = new int[(int)pixelSize];

            // プレビューフレームを取得
            _camera.GetPreviewBufferArgb32(pixelData);

            //Imageに表示
            // バッファーをWriteableBitmapへコピー
            var bmp = new WriteableBitmap((int)_camera.PreviewResolution.Width, (int)_camera.PreviewResolution.Height);
            pixelData.CopyTo(bmp.Pixels, 0);

            using (var store = IsolatedStorageFile.GetUserStoreForApplication())
            using (var strm = store.CreateFile("preview.jpg"))
            {
                bmp.SaveJpeg(strm, bmp.PixelWidth, bmp.PixelHeight, 0, 100);
            }
        }

        private void ImageButton_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            // プレビューフレームを取得するバッファーを確保する
            double pixelSize = _camera.PreviewResolution.Width * _camera.PreviewResolution.Height;
            var pixelData = new int[(int)pixelSize];

            // プレビューフレームを取得
            _camera.GetPreviewBufferArgb32(pixelData);

            //Imageに表示
            // バッファーをWriteableBitmapへコピー
            var bmp = new WriteableBitmap((int)_camera.PreviewResolution.Width, (int)_camera.PreviewResolution.Height);
            pixelData.CopyTo(bmp.Pixels, 0);

            ImagePreview.Source = bmp;
        }
    }
}

